"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));

// src/plugin.ts
var import_esbuild = __toESM(require("esbuild"));
var import_fs = __toESM(require("fs"));
var import_path = __toESM(require("path"));

// src/dynamodb.ts
var import_client_dynamodb = require("@aws-sdk/client-dynamodb");
var import_util_dynamodb = require("@aws-sdk/util-dynamodb");

// src/serverless.ts
var sls;
var setServerless = (serverless) => {
  sls = serverless;
};
var useServerless = () => {
  return sls;
};

// src/utils.ts
function lazy(callback) {
  let loaded = false;
  let result;
  return () => {
    if (!loaded) {
      result = callback();
      loaded = true;
    }
    return result;
  };
}
function logicalName(name) {
  return toPascalCase(name);
}
function toPascalCase(string) {
  return `${string}`.toLowerCase().replace(new RegExp(/[-_]+/, "g"), " ").replace(new RegExp(/[^\w\s]/, "g"), "").replace(
    new RegExp(/\s+(.)(\w*)/, "g"),
    ($1, $2, $3) => `${$2.toUpperCase() + $3}`
  ).replace(new RegExp(/\w/), (s) => s.toUpperCase());
}

// src/credentials.ts
var useAWSProvider = lazy(() => {
  const sls2 = useServerless();
  const provider = sls2.getProvider("aws");
  if (!provider) {
    throw new Error("serverless-live-lambda: only AWS is supported");
  }
  return provider;
});
var useAWSCredentials = lazy(async () => {
  const provider = useAWSProvider();
  const { credentials } = provider.getCredentials();
  return credentials;
});
var useClientCache = lazy(() => /* @__PURE__ */ new Map());
function useAWSClient(client, force = false) {
  const cache = useClientCache();
  const existing = cache.get(client.name);
  if (existing && !force)
    return existing;
  const [provider, credentials] = [useAWSProvider(), useAWSCredentials()];
  const result = new client({
    region: provider.getRegion(),
    credentials
  });
  cache.set(client.name, result);
  return result;
}

// src/errors.ts
var VisibleError = class extends Error {
  constructor(...message) {
    super(message.join("\n"));
  }
};

// src/dynamodb.ts
async function describeTable(table) {
  const client = useAWSClient(import_client_dynamodb.DynamoDBClient);
  const input = new import_client_dynamodb.DescribeTableCommand({
    TableName: table
  });
  const response = await client.send(input);
  if (!response.Table) {
    throw new VisibleError(`${table} resource not found`);
  }
  let hashKey = "";
  let rangeKey;
  for (const key of response.Table.KeySchema || []) {
    if (key.KeyType === "HASH") {
      hashKey = key.AttributeName;
    }
    if (key.KeyType === "RANGE") {
      rangeKey = key.AttributeName;
    }
  }
  for (const attr of response.Table.AttributeDefinitions || []) {
    if ((attr.AttributeName === hashKey || attr.AttributeName === rangeKey) && attr.AttributeType !== "S") {
      throw new VisibleError("hash key and range key must be a string");
    }
  }
  return {
    tableName: table,
    keySchema: {
      hashKey,
      rangeKey
    }
  };
}

// src/plugin.ts
var ServerlessDynamoDbOutputs = class {
  constructor(serverless, _options, { log }) {
    this.hooks = {
      "package:initialize": this.beforePackage.bind(this)
    };
    this.serverless = serverless;
    this.log = log;
    setServerless(serverless);
    this.serverless.configSchemaHandler.defineCustomProperties({
      type: "object",
      properties: {
        dynamodbExport: {
          type: "object",
          properties: {
            tableName: { type: "string" },
            params: {
              anyOf: [
                {
                  type: "object",
                  additionalProperties: {
                    oneOf: [{ type: "string" }, { type: "object" }]
                  }
                },
                {
                  type: "array",
                  items: {
                    type: "object",
                    properties: {
                      hashKey: { type: "string" },
                      rangeKey: { type: "string" },
                      value: {
                        oneOf: [{ type: "string" }, { type: "object" }]
                      }
                    },
                    required: ["hashKey", "value"]
                  }
                }
              ]
            }
          },
          required: ["tableName"]
        }
      }
    });
  }
  async beforePackage() {
    const exports2 = this.serverless.service.custom?.dynamodbExport;
    if (!exports2) {
      this.log.debug("no exports");
      return;
    }
    const { tableName, params } = exports2;
    const template = this.serverless.service.provider.compiledCloudFormationTemplate;
    const {
      functionLogicalID: customResourceLogicalId,
      resources: customResources
    } = this.createPutVariableLambdaFunctionResource();
    template.Resources = {
      ...template.Resources,
      ...customResources
    };
    const table = await describeTable(tableName);
    if (Array.isArray(params)) {
      for (const param of params) {
        if (!param.hashKey || !param.value) {
          this.log.error(`invalid schema ${JSON.stringify(param)}`);
          continue;
        }
        if (table.keySchema.rangeKey && !param.rangeKey) {
          throw new Error(
            `Invalid table key schema. The range key "${table.keySchema.rangeKey}" doesn't provided in serverless config"`
          );
        }
        const logicalId = "DynamoDbOutputVariable" + logicalName(param.hashKey) + (param?.rangeKey ? logicalName(param.rangeKey) : "");
        template.Resources[logicalId] = {
          Type: "Custom::DynamoDbOutputVariable",
          DependsOn: customResourceLogicalId,
          Properties: {
            ServiceToken: {
              "Fn::GetAtt": [customResourceLogicalId, "Arn"]
            },
            TableName: tableName,
            HashKey: table.keySchema.hashKey,
            RangeKey: table.keySchema.rangeKey,
            Item: {
              [table.keySchema.hashKey]: param.hashKey,
              ...table.keySchema.rangeKey ? {
                [table.keySchema.rangeKey]: param.rangeKey
              } : {},
              value: param.value
            }
          }
        };
      }
      return;
    }
    if (typeof params === "object") {
      for (const [name, value] of Object.entries(params)) {
        const logicalId = "DynamoDbOutputVariable" + logicalName(name);
        template.Resources[logicalId] = {
          Type: "Custom::DynamoDbOutputVariable",
          DependsOn: customResourceLogicalId,
          Properties: {
            ServiceToken: {
              "Fn::GetAtt": [customResourceLogicalId, "Arn"]
            },
            TableName: tableName,
            HashKey: table.keySchema.hashKey,
            RangeKey: table.keySchema.rangeKey,
            Item: {
              [table.keySchema.hashKey]: name,
              ...table.keySchema.rangeKey ? {
                [table.keySchema.rangeKey]: name
              } : {},
              value
            }
          }
        };
      }
    }
  }
  createPutVariableLambdaFunctionResource() {
    const entryPoint = "../assets/put-variable";
    const distFile = import_path.default.join(
      __dirname,
      `${this.serverless.utils.generateShortId(10)}.js`
    );
    import_esbuild.default.buildSync({
      entryPoints: [entryPoint],
      bundle: true,
      packages: "external",
      platform: "node",
      outfile: distFile
    });
    const code = import_fs.default.readFileSync(distFile, { encoding: "utf-8" });
    import_fs.default.rmSync(distFile);
    const functionLogicalID = "CustomResourcePutVariableLambdaFunction";
    const result = {
      functionLogicalID,
      resources: {
        CustomResourcePutVariableLambdaRole: {
          Type: "AWS::IAM::Role",
          Properties: {
            AssumeRolePolicyDocument: {
              Version: "2012-10-17",
              Statement: [
                {
                  Effect: "Allow",
                  Principal: {
                    Service: "lambda.amazonaws.com"
                  },
                  Action: "sts:AssumeRole"
                }
              ]
            },
            Policies: [
              {
                PolicyName: "default",
                PolicyDocument: {
                  Version: "2012-10-17",
                  Statement: [
                    {
                      Effect: "Allow",
                      Action: [
                        "logs:CreateLogGroup",
                        "logs:CreateLogStream",
                        "logs:PutLogEvents"
                      ],
                      Resource: "*"
                    },
                    {
                      Effect: "Allow",
                      Action: [
                        "dynamodb:PutItem",
                        "dynamodb:DescribeTable",
                        "dynamodb:DeleteItem"
                      ],
                      Resource: ["arn:aws:dynamodb:*:*:table/*"]
                    }
                  ]
                }
              }
            ]
          }
        },
        [functionLogicalID]: {
          Type: "AWS::Lambda::Function",
          Properties: {
            Handler: "index.handler",
            Runtime: "nodejs18.x",
            Role: {
              "Fn::GetAtt": ["CustomResourcePutVariableLambdaRole", "Arn"]
            },
            MemorySize: 128,
            Code: {
              ZipFile: code
            }
          }
        }
      }
    };
    return result;
  }
};
var plugin_default = ServerlessDynamoDbOutputs;

// src/index.ts
module.exports = plugin_default;
//# sourceMappingURL=index.js.map
